function measureMag(filename, num_cycles, group, element)

%Creating an array with the number of line-pairs per cycle for each element
%of the USAF resolution test.  The array is of the form
%USAF_target(element, group)
USAF_target = [0.25, 0.5, 1, 2, 4, 8, 16, 32, 64, 128, 256, 512;
0.28, 0.561, 1.12, 2.24, 4.49, 8.98, 17.95, 36, 71.8, 144, 287, 575;
0.315, 0.63, 1.26, 2.52, 5.04, 10.1, 20.16, 40.3, 80.6, 161, 323, 645;
0.353, 0.707, 1.41, 2.83, 5.66, 11.3, 22.62, 45.3, 90.5, 181, 362, 0;
0.397, 0.793, 1.59, 3.17, 6.35, 12.7, 25.39, 50.8, 102, 203, 406, 0;
0.445, 0.891, 1.78, 3.56, 7.13, 14.3, 28.5, 57, 114, 228, 456, 0];

%Calculating the distance per cycle for the specified element
distance_per_cycle = 1E-3/USAF_target(element, group+3);

%Loading data to the variable "profile"
profile = imread(filename);

%Calculating the intensity threshold, to create a black and white image
threshold = mean((max(max(profile(:,:)))-min(min(profile(:,:))))/2+min...
    (min(profile(:,:))));

%Initialising the results table
results(1:size(profile,2), 1:5) = 0;

%For each cross-section of the image the top and bottom rows of the cycle
%are identified and stored in the results table, along with an analysis of
%the distance for each cross-section. Column 1 is the line number, column 2
%is the bottom row of the cycle, column 3 is the top row of the cycle, 
%column 4 is the number of rows per cycle and column 5 is the number of 
%pixels per cycle.
for i = 1:size(profile,2)
    below_threshold = find(profile(:,i)<threshold);
    bottom = min(below_threshold);
    
    above_threshold = find(profile(:,i)>=threshold);
    top = max(above_threshold);
    
    try
        results(i,1) = i;
        results(i,2) = bottom;
        results(i,3) = top;
        results(i,4) = (top-bottom)/num_cycles;
        results(i,5) = (num_cycles*distance_per_cycle)/(top-bottom);
        
    catch
        results(i,1) = i;
        results(i,2) = NaN;
        results(i,3) = NaN;
        results(i,4) = NaN;
        results(i,5) = NaN;
        
    end
    
end

%Calculating the mean and mode distances per pixel for the image and
%displaying them in the command window
distance_per_px = mean(results(:,4));
distance_per_px_stdev = std(results(:,4));

distance_per_mpx = mean(results(:,5));
distance_per_mpx_stdev = std(results(:,5));

diary(strcat([filename, '_output.txt']));
disp(strcat(['Mean px/lp = ', num2str(distance_per_px), ' px/lp +/- ', ...
    num2str(distance_per_px_stdev), ' px/lp']))
disp(strcat(['Mode px/lp= ', num2str(mode(results(:,4))), ' px/lp']))
disp(strcat(['Mean m/px = ', num2str(distance_per_mpx), ' m/px +/- ', ...
    num2str(distance_per_mpx_stdev), ' m/px']))
disp(strcat(['Mode m/px= ', num2str(mode(results(:,5))), ' m/px']))
diary off

%Showing a figure of the resolution test, with the top line highlighted in
%blue and the bottom line highlighted in red.
figure()
imshow(profile)
hold on
plot(results(:,1),results(:,2), 'bo')
plot(results(:,1),results(:,3), 'ro')

%Saving the results to an Excel file
xlswrite(strcat(filename, '.xls'), results);

disp ' '
