clear all
%% Setting some variables
t = 0; %The current time (in seconds).  This will increment in values of dt
%after each iteration

total_time = 20; %The end time (in seconds)
dt = 0.005E-3; %The discrete step time.  From Velthuis
L_ext = 6.22E-6; %The end-end tether length at the given force, F.  This comes
%from the Worm-Like Chain model
F = 2.9E-12; %The force acting on the tether.  Obtained from the
%worm-like chain model for an end to end tether extension 6.49 um;
R = 0.595E-6; %The radius of the bead.  In this case it's 1.4um as I was
%using the M280 beads.
eta = 0.001; %The dynamic viscosity of the fluid the bead is in.  This has a
%value of 0.001 kg/ms from the Velthuis paper supplementary material
kB = 1.38E-23; %Boltzmann constant
T = 293; %Temperature in Kelvin
D = (kB*T)/(6*pi*eta*R); %The diffusion coefficient of the bead.
n_runs = 10; %The number of times to run the simulation

f_name_input = inputdlg('Enter filename');
f_name = f_name_input{1};

for run_i = 1:n_runs
    diary(strcat([f_name, ' Run ', num2str(run_i), '.txt']));
        
    disp(strcat(['Filename: ', f_name, ' Run ', num2str(run_i)]));
    disp(strcat(['Total time = ', num2str(total_time), ' s']));
    disp(strcat(['Time step = ', num2str(dt), ' s']));
    disp(strcat(['Input force = ', num2str(F*1E12), ' pN']));
    disp(strcat(['Tether extension = ', num2str(L_ext), ' m']));
    disp(strcat(['Microsphere radius = ', num2str(R), ' m']));
    disp(strcat(['Fluid viscosity = ', num2str(eta), ' kg/ms']));
    disp(strcat(['Temperature = ', num2str(T), ' K']));
    disp(strcat(['Diffusion coefficient = ', num2str(D), ' m^2/s']));
    disp(' ')
    
    real_data(ceil(total_time/dt)-1,1:2) = 0;
    
    %% Running the model
    real_data(1,1) = 0; %Initial time is zero
    real_data(1,2) = 0; %Initial displacement is zero
    
    for i = 2:ceil(total_time/dt)
        real_data(i,1) =  real_data(i-1,1) + dt;
        real_data(i,2) =  real_data(i-1,2) + (F/(L_ext))*((-real_data(i-1,2)*D)/(kB*T))*dt + sqrt(2*D*dt)*randn(1,1); %displacement parallel to field lines
    end
    
    var_real = var(real_data(:,2));
    F_real = (kB*T*L_ext)/var_real;
       
    %% Displaying results
    disp(strcat(['Measured simulation variance = ', num2str(var_real), ' m']));
    disp(strcat(['Measured simulation force = ', num2str(F_real*1E12), ' pN']));
    disp(strcat(['Maximum measured simulation displacement = ', num2str(max(abs(real_data(:,2)))), ' m']));
    disp(' ');
    
    [data_a data_b] = simulateCamera(strcat([f_name, ' Run ', num2str(run_i), '_Watec_902DM2S']), real_data, dt, kB, T, L_ext, 1/(100), 25, 1);
    [data_c data_d] = simulateCamera(strcat([f_name, ' Run ', num2str(run_i), '_TIS_DMK 22BUC03_(USB2 CMOS)']), real_data, dt, kB, T, L_ext, 1/(200), 60, 0);

    output_table(run_i,1) = var_real;
    output_table(run_i,2) = data_a; %Measured variance
    output_table(run_i,3) = data_b; %Measured variance (interlaced)
    output_table(run_i,4) = data_c; %Measured force
    output_table(run_i,5) = data_d; %Measured force (interlaced)
    
    diary off
    disp(' ')
    clear real_data
    
end

xlswrite(strcat([f_name, '_full_variance_data']), output_table)