function [output_table, n_table] = mcdDiffusionCore(box_size, D, total_time_s, dt_s, R, L, field_depth, intensity0, d, mesh_wobble_cnd, rotation_cnd, plot_cnd, boundary_pass_threshold, n_table, vac_diff_cnd, membrane_table, R_BtuB, R_OmpA, R_OmpF, local_range)

if R ~= 0
    curvature_cnd = 1;
else
    curvature_cnd = 0;
end

if box_size ~= 0
    restricted_movement_cnd = 1;
else
    restricted_movement_cnd = 0;
end

dt_s_i = 0;  %The current iteration in terms of time
dt_s_table_i = 1;   %The current iteration as an integer
out_of_field_cnd = 0; %Set to zero until the particle moves further back than the evanescent field extends.  Only comes into play when there is a radius of curvature
boundary_pass_i = 2;
boundary_pass_time(1) = 1;
output_table = zeros(round(total_time_s/dt_s)+1,4);

if vac_diff_cnd == 0
    [dx, dy] = mcdRandomStart(R, field_depth, L);
else
    [~, ~, x_max, y_max] = mcdRandomStart(R, field_depth, L);
    [dx, dy] = mcdPositionBtuB(membrane_table, R_OmpA, R_OmpF, R_BtuB, x_max*2, y_max*2);
    dx_0 = dx;
    dy_0 = dy;
    OmpF_table = membrane_table(membrane_table(:,4)==1,:);
    OmpA_table = membrane_table(membrane_table(:,4)==2,:);
    boundary = (R_BtuB + 2*R_OmpF + R_OmpF/tan(30*pi/180) - R_OmpF- R_OmpF*tan(30*pi/180));
    OmpF_table_local = mcdCreateLocalTable(OmpF_table, dx, dy, boundary, local_range);
    OmpA_table_local = mcdCreateLocalTable(OmpA_table, dx, dy, boundary, local_range);
end

if restricted_movement_cnd == 1 %This section executes if the particles are to be subject to restricted motion in the square mesh
    if mesh_wobble_cnd == 1 %This section randomly moves the mesh origin, which simulates random bacterial rotations around its long axis
        dx_mesh = 1.1*rand(1,1)*box_size*0.5*mcdSignGeneration;
        dy_mesh = 1.1*rand(1,1)*box_size*0.5*mcdSignGeneration;
    else %If the mesh is set to be fixed the mesh origin is always set to "0"
        dx_mesh = 0;
        dy_mesh = 0;
    end
    
    dx = dx + dx_mesh;    %This shifts the positions by a fixed amount (the same for all attempts for a single particle) to simulate random mesh locations on the surface of the bacteria
    dy = dy + dy_mesh;
    [previous_box] = mcdMeshFind(dx, dy, box_size); %Calculates the coordinates of the mesh square the particle was in at the previous step
    dx = dx - dx_mesh;    %Now the current mesh box has been found the positions are replaced to be centred around the origin
    dy = dy - dy_mesh;
end

%Implementing the rotation parameters
if rotation_cnd == 1  %Here the bacteria long axis is aligned with the y-axis
    rotation = rand(1,1)*2*pi;  %Disabling the row above and using this sets a different orientation for each particle
end

for particle_run = 1:(total_time_s/dt_s)+1
    dx_previous = dx; %These will be used later to reassign dx and dy for addition to the next step of the random walk
    dy_previous = dy;
    
    if dt_s_table_i ~= 1  %For the first step the location has already been calculated, so it is skipped
        if vac_diff_cnd == 0
            [dx dy] = mcdConstStepDiff(D,dt_s, dx, dy);
        else
            %Checking to see if the particle has moved further than the
            %local range for which OmpF and OmpA positions are held.  If it
            %has moved further, the local tables need to be recalculated.
            if sqrt((dx-dx_0)^2+(dy-dy_0)^2) > local_range
                boundary = (R_BtuB + 2*R_OmpF + R_OmpF/tan(30*pi/180) - R_OmpF- R_OmpF*tan(30*pi/180));
                OmpF_table_local = mcdCreateLocalTable(OmpF_table, dx, dy, boundary, local_range);
                OmpA_table_local = mcdCreateLocalTable(OmpA_table, dx, dy, boundary, local_range);
                dx_0 = dx;
                dy_0 = dy;
            end
            
            [dx dy] = mcdDiffuseBtuB(OmpF_table_local, OmpA_table_local, dx, dy, D, dt_s, R_OmpA, R_OmpF, R_BtuB, x_max*2, y_max*2);
        end
    end
    
    if restricted_movement_cnd == 1 %This section executes if the particles are to be subject to restricted motion in the square mesh
        [dx dy previous_box] = mcdRestrictedMovement(dx, dy, dx_previous, dy_previous, dx_mesh, dy_mesh, previous_box, box_size, boundary_pass_threshold);
    end
    
    if rotation_cnd == 1
        [dx dy] = mcdCellRotation(dx, dy, rotation);
    end
    
    if out_of_field_cnd == 0  %If the particle goes out of the evanescent field this will be set to 1 (then 2 by another loop), so this won't evaluate
        if curvature_cnd == 1
            [dxc dyc dzc out_of_field_cnd] = mcdCellCurvature(dx, dy, L, R, field_depth, out_of_field_cnd);
        else
            dxc = dx;
            dyc = dy;
            dzc = 0;
        end
    end
    
    if rotation_cnd == 1 %Rotating the coordinates back
        [dx dy] = mcdCellRotation(dx, dy, -rotation);
        [dxc dyc] = mcdCellRotation(dxc, dyc, -rotation);
    end
    
    if out_of_field_cnd ~= 0 %For each time step and particle within a run, the coordinates of each particle are recorded in "output_table"
        output_table(dt_s_table_i,1:4) = NaN; %If the particle moved out the TIRFM field, the relevant cell contains a "NaN" value
        
    else %If the particle is still within the TIRFM field, the "x", "y" and "z" coordinates are recorded along with the intensity
        output_table(dt_s_table_i,1) = dxc;
        output_table(dt_s_table_i,2) = dyc;
        output_table(dt_s_table_i,3) = dzc;
        output_table(dt_s_table_i,4) = intensity0*exp(-(output_table(dt_s_table_i,3))/d);
        
    end
    
    if out_of_field_cnd == 0 %Each time a particle successfully 'survives' a time step without going out of the TIRFM field, it adds a counter to "n_table".  For calculating standard error, the number of particles needs to be known.  This value is wiped after each, run so is only useful when using a single run.
        n_table(dt_s_table_i) = n_table(dt_s_table_i) + 1;
    end
    
    dt_s_i = dt_s_i + dt_s;
    dt_s_table_i = dt_s_table_i + 1;
    
end

if plot_cnd == 1
    mcdPlotDiffusion(boundary_pass_time, boundary_pass_i, output_table, R, dt_s_table_i);
end