%PARTICLETRACKING4.M

%This function is an extended version of "particledetect.m".  It analyses a
%sequence of image files, detecting particles in each one, then identifying
%the same particle in different frames.  Using this particle tracking,
%particle displacements can be calculated and the data can be displayed as
%video or a static image.

%INPUTS:
%"gauss_cnd" determines how particles are detected.  "0" detects particle
%centres as the pixel with the highest intensity.  "1" fits 1D Gaussian
%curves to each row of pixels and averages the centres. "2" fits a 2D
%Gaussian curve to each particle.  "3" is a special case for "doughnut-
%shaped" particles, which fits a pair of 2D Gaussian curves

%"gauus_period" is a parameter required for "gauss_cnd = 1", which
%determines the accuracy with which the Gaussian curves are fitted

%"max_gauss_evals" is a parameter required for "gauss_cnd = 2 and 3",
%which determines the maximum number of evaluations the minimsation
%function will perform.  Decreasing this speeds up code execution, but
%decreases particle detection accuracy.

%"allow_miss_percentage" is the percentage of frames a particle may be
%absent from before it is discarded.  This can be increased for videos
%where particles rapidly appear and disappear

%"movie_cnd" determines if the tracked particles are displayed in a
%video after particle tracking has executed

%"linedraw_cnd" determines if the tracked particles are displayed in an
%image after particle tracking has executed


function mtParticleTracking(filename, tether_length, MaxFunEvals, TolFun, TolX, allow_miss_percentage, movie_cnd, linedraw_cnd)

%Initialising "tracked_table", which will later be populated with
%individual particle locations for each frame.
tracked_table = zeros(1);
R = 0; %Placeholder for R

%% Loading the video to be analysed
dot_loc = strfind(filename, '.');
root_filename = filename(1:dot_loc(length(dot_loc))-1);
ext = filename(dot_loc(length(dot_loc))+1:length(filename));

if exist(filename) == 0
    file_not_found_warn = warndlg(strcat(['File "', filename, '" not found']));
    uiwait(file_not_found_warn);
    return
end

video_object = VideoReader(filename);
n_frames = video_object.NumberOfFrames;
data = read(video_object);

b = imread(strcat([root_filename, '_Background.png']));

start_peak_table = xlsread(strcat([root_filename, '_Start Positions.xls']));
mask_R = ceil(max(start_peak_table(:,4))*2);

load(strcat(root_filename, '_Parameters.mat'));

%This deletes the previously created tracked particles table if it exists
if exist(strcat(root_filename, '_Locations', '.xls')) == 2
    delete(strcat(root_filename, '_Locations', '.xls'))
end

%This shows the filename of the file currently being executed
disp (['Evaluating "', filename, '"'])

%For many images, particle tracking can take quite a while, so a progress
%bar is displayed.  Here, this is initialised.
progress = waitbar(0, 'Tracking particles...');

peak_table = start_peak_table;
%The following loop performs the main execution.  Each iteration the next
%image in the set is analysed.
for filenumber = 1:(n_frames)
    waitbar(filenumber/n_frames);
    temp_data = rgb2gray(data(:,:,:,filenumber));
    temp_data = b - temp_data;
    size_raw_data = size(temp_data);
    
    %Applying a mask to make the border of the image black.  This prevents
    %pixels outside the image being accessed during gaussian fitting.
    raw_data(1:size(temp_data,1)+2*mask_R+1, 1:size(temp_data,2)+2*mask_R+1)=uint8(0);
    
    for i = 1:size_raw_data(1,1)
        for j = 1:size_raw_data(1,2)
            raw_data(i+mask_R,j+mask_R) = temp_data(i,j);
            
        end
    end
    
    %Checks some peaks have been detected.  If not, "peak_table" will be
    %empty and the script will terminate
    size_peak_table = size(peak_table);
    if exist ('peak_table', 'var') == 0;
        return
        
    elseif size_peak_table(1,1) == 0
        return
        
    end
    
    [peak_table] = mtImRegisterFit(peak_table, raw_data, R, size_peak_table(1,1));
    
    %This takes the detected particles for the current image and compares
    %them to previously detected particles.  All particles are placed in
    %"tracked_table" with an assigned particle number, which is
    %consistent for single particles in each image.
    [tracked_table] = mtPeakFollow(filenumber, peak_table, ...
        disappear_threshold, drift_threshold, tracked_table);
    
    %Clearing variables for the next iteration with the next frame in the
    %set
    clear temp_data raw_data
    
end

%Having the video loaded to RAM takes a lot of space.  It's not needed here
%any more, so can be closed
clear data

%Close the "Subtracting background" progress bar
try
    close(progress)
end

%This section deletes any columns in "tracked_table" with more
%missing rows than "allow_miss_percentage".  First, the width of
%"tracked_table" is calculated, as this determines the number of times the
%system must evaluate.
tracked_table_size = size(tracked_table);

%Only every other column is analysed for zeroes, because a "missing"
%particle won't have both x and y coordinates.  The loop evaluates the
%table from the right, then moves left, which should prevent problems of
%missed columns as columns are removed
for col = tracked_table_size(1,2):-3:4
    %The "find" function is used to find the number of rows in the current
    %column, which have "0".  The number is counted as the size of the
    %table
    missed_rows = find (tracked_table(:,col)==0);
    number_missed = size(missed_rows);
    
    %If the number missed is greater than "allow_miss_percentage" both
    %columns relating to that particle are deleted
    if (number_missed(1)/tracked_table_size(1,1))*100 > ...
            allow_miss_percentage
        tracked_table(:,col-2:col) = [];
    end
    
end

%Zeroes are a problem when using data exported to Excel, so 0s are replaced
%with 'NaN', which are empty cells (Not a Number)
tracked_table(find(tracked_table == 0)) = NaN;

%Putting headers on the columns of 'tracked_table', which then go into the
%cell "tracked_table".  Initially, a header row of the same width as
%"tracked_table" is created.  The two tables are then concatenated
%together.  For text to be entered to the table it must be converted to
%"cell" format, which is achieved with "num2cell"
tracked_table_size = size(tracked_table);
tracked_table = num2cell([1:tracked_table_size(1,2);tracked_table]);

%Assigning the first column of the header row as "Frame"
tracked_table(1,1) = {'Frame'};

%The columns are numbered "Xn" and "Yn", where "n" is the current particle
current_particle = 1;

%Going across the table the labels "Xn" and "Yn" are added to the first row
%of each column.  At the end of each iteration the current particle number
%increases by "1"
for i = 2:3:tracked_table_size(1,2)
    tracked_table(1,(i+1)) = {strcat('X', num2str(current_particle))};
    tracked_table(1,i) = {strcat('Y', num2str(current_particle))};
    tracked_table(1,i+2) = {strcat('Residual ', num2str(current_particle))};
    current_particle = current_particle + 1;
end

%The tracked particles table is automatically saved, but the data can
%become distorted if a file already exists in the directory.  Therefore,
%the system checks for an existing tracked particle file and deletes it if
%there is one.
if exist (strcat(root_filename,'_Locations', '.xls'), 'var') == 2;
    delete(strcat(root_filename,'_Locations', '.xls'));
end

%This writes the tracked particle table to an Excel file with the suffix
%"locations".
xlswrite(strcat(root_filename,'_Locations', '.xls'), tracked_table);

%To quickly inform the user of the particle tracking result, the number of
%tracked particles is displayed in the MATLAB command window.
size_tracked_table = size(tracked_table);
disp ([num2str((size_tracked_table(1,2)-1)/3), ' particles tracked']);

mtAnalyseTrackedParticles(filename, tether_length);

%If the user requested the particle displacements be displayed as a video,
%this condition executes, which runs the "makemovie" function
if movie_cnd == 1
    mtMakeMovie(filename);
    
end

%If the user requested the particle displacements be displayed on an image,
%this condition executes, which runs the "linedraw" function
if linedraw_cnd == 1
    mtLineDraw(filename)
    
end