%PEAKFOLLOW2.M

%This identifies detected particles in adjacent frames as corresponding to
%the same particle if they fall within certain thresholds.  These
%thresholds are drift_threshold, which specifies the number of pixels a
%particle centre can move in subsequent images and disappear_threshold,
%which specifies the number of frames for which a particle can be
%undetected before being classed as a new particle.  Individual particle
%locations are stored in peak_table as column pairs of X and Y coordinates
%as a function of frame number (going down the column).

%INPUTS
%"filenumber" is used by particle detect to specify the image in the set to
%be analysed

%"peak_table" is a table of detected particle locations.  It has columns:
%1 = peak identifier, 2 = row of peak, 3 = column of peak, 4 = maximum
%intensity of peak (used for fitting gaussian profile)

%"disappear_threshold" is the number of frames a particle can go undetected
%while still being linked to a particle in previous frames

%"drift_threshold" determines the amount a particle centre can drift
%between adjacent frames of a video

%"tracked_table" is an array containing the x and y coordinates for the
%tracked particles.  Each tracked particle occupies two columns (x and y
%coordinates) with a new particle taking the next pair of columns

%OUTPUTS
%"tracked_table" is an array containing the x and y coordinates for the
%tracked particles.  Each tracked particle occupies two columns (x and y
%coordinates) with a new particle taking the next pair of columns


function [tracked_table] = mtPeakFollow(filenumber, peak_table, ...
    disappear_threshold, drift_increment, tracked_table)

%The number of detected particle events
size_peak_table = size(peak_table);
n = size_peak_table(1,1);


if (filenumber == 1) %This part puts the values from the first 'peak_table'
    %in 'tracked_table', as these will all relate to different particles
    
    %Column 1 is the image file number (frame number)
    tracked_table(filenumber,1) = filenumber;
    
    %Column 2 (and every 3rd column thereon) is the x-coordinate of a
    %particle
    tracked_table(filenumber,2:3:(n*3)-1) = peak_table(1:n,2);
    
    %Column 3 (and every 3rd column thereon) is the y-coordinate of a
    %particle
    tracked_table(filenumber,3:3:(n*3)) = peak_table(1:n,3);
    
    %Column 4 (and every 3rd column thereon) is the residual from
    %fminsearch
    tracked_table(filenumber,4:3:(n*3)+1) = peak_table(1:n,5);
    
else %For all subsequent images, this condition executes, which compares
    %each set of particle centres to the previous 'n' tables to see if they
    %are the same particles.  If not, put them in a new column
    
    %This loop goes down the rows of peak_table and compares the current
    %particle in "peak_table" to the particles already in "tracked_table"
    %to find particles at similar locations within a user-defined time
    
    %Adding the current filenumber (frame number) to the left-most column
    tracked_table(filenumber,1) = filenumber;
    
    for i = 1:n
        %Calculating the current number of columns, which gives an
        %indication of the number of different detected particles
        tracked_table_size = size(tracked_table);
        cols = tracked_table_size(1,2);
        
        %Counts up how many consecutive times a particle hasn't been 
        %matched to a particle already in the table.
        missed_i = 1;   
                        
        %This value is set to "0" by default.  If the current particle is 
        %linked to an existing particle in "tracked_table" it is set to
        %"1".  If no such particle is found it remains as "0" and the
        %particle is assigned to a new pair of columns
        new_particle = 0;
        
        %Once the current particle has been linked to an existing particle 
        %this value changes to "1" and the system exists the comparing
        %particles loop
        linked_condition = 0;
        
        %While the system hasn't found a similar particle to the current
        %particle this loop executes
        while linked_condition ~= 1
            drift_threshold = drift_increment*missed_i;

            %Incase the system tries to regress to a row not in the table
            %(it starts at the bottom and works up) this try-catch
            %condition activates.  
            try
                %The current row of particles for the current particle to
                %be tested against.  This starts at the row below the
                %current frame and works up towards the top row (first
                %frame) as "missed_i" increases by "1" on each iteration of
                %the loop
                row = filenumber - missed_i;
                
                %For the row currently being interrogated the distances
                %between the current particle and all particles in that row
                %are calculated using Pythagoras' Theorem.  The minimum is
                %then taken as the closest distance.
                min_res = min(sqrt(((tracked_table(row,2:3:(cols-2))-...
                    peak_table(i,2)).^2)+((tracked_table(row,3:3:cols-1)-...
                    peak_table(i,3)).^2)));
                
                %If the previously calculated minimum distance is below
                %"drift_threshold" it would appear the current particle is
                %the same as the particle, which gave that minimum distance
                %and this condition executes
                if min_res < drift_threshold
                    %The particle which yielded that minimum distance is
                    %found and the variable "new_col" is set to the first
                    %(x) column of that particle
                    new_col = find((sqrt(((tracked_table(row,2:3:(cols-...
                        2))-peak_table(i,2)).^2)+((tracked_table(row,...
                        3:3:cols-1)-peak_table(i,3)).^2))) == min_res);
                    
                    %"linked_condition" is set to "1" to indicate a
                    %linked particle has been located
                    linked_condition = 1;
                    
                else %If the minimum distance is greater than the 
                    %user-defined "drift_threshold" this condition executes
                    
                    %"missed_i" is increased by "1" and "linked_condition"
                    %is kept at "0", so the main loop will restart, but for
                    %the next row up in the table
                    missed_i = missed_i + 1;
                    
                    %If more frames than "disappear_threshold" have passed
                    %it is assumed there was no previous particle
                    %detectable and the system is told to assign the
                    %current particle to a new pair of columns in
                    %"tracked_table"
                    if missed_i >= disappear_threshold
                        new_particle = 1;
                        linked_condition = 1;
                    end
                    
                end
               
            catch %In the event the system tries to access a row outside of
                %"tracked_table" no suitable particle has been detected, so
                %the system is told to assign the current particle to a new
                %pair of columns in "tracked_table"
                new_particle = 1;
                linked_condition = 1;
            end
        end
        
        %If "new_particle" has been set to "1" the particle hasn't been
        %linked to a particle already in the table, so it is placed in the
        %next two available columns in "tracked_table", which is set here
        if new_particle == 1
            new_col = (cols-1)/3+1;
            
        end
    
        %The system will have assigned the columns for the current particle
        %to be placed in, so here it is added to "tracked_table" at that
        %location
        tracked_table(filenumber, (new_col*3)-1) = peak_table(i,2);
        tracked_table(filenumber, (new_col*3)) = peak_table(i,3);
        tracked_table(filenumber, (new_col*3)+1) = peak_table(i,5);
        
    end
    
end
